/*    
       This software may be used and distributed according to the terms of
       the GNU General Public License (GPL), incorporated herein by
       reference.  Drivers based on this skeleton fall under the GPL and
       must retain the authorship (implicit copyright) notice.
 
       This program is distributed in the hope that it will be useful, but
       WITHOUT ANY WARRANTY; without even the implied warranty of
       MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
       General Public License for more details.
 */
package beaconconfig;

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import java.util.*;

class SketchPanel extends JPanel 
    implements ActionListener
{
    private SketchControl sketcher;

    protected JButton eraseAll;
    protected JRadioButton lineControl;
    protected JRadioButton rectControl;
    protected JRadioButton circControl;
    protected JRadioButton penUpControl;
    protected JRadioButton eraseControl;
    private JLabel penDn;

    
    private static String lineControlLabel = "Line";
    private static String rectControlLabel = "Rect";
    private static String circControlLabel = "Circ";
    private static String penUpControlLabel = "PenUp";
    //private static String eraseControlLabel = "Erase";

    public SketchPanel(SketchControl s) 
    {
	//((FlowLayout)getLayout()).setHgap(0);
	setLayout(new GridLayout(1, 5));
	sketcher = s;
	
	eraseAll = addButton("EraseAll");
	penUpControl = addRadioButton(penUpControlLabel);
	//eraseControl = addRadioButton(eraseControlLabel);
	penDn = new JLabel("PenDn:", JLabel.CENTER);
	//penDn.setMaximumSize(new Dimension(20,17));
	//System.out.println(penDn.getMaximumSize());
	add(penDn);
	lineControl = addRadioButton(lineControlLabel);
	rectControl = addRadioButton(rectControlLabel);
	circControl = addRadioButton(circControlLabel);
	reset();
    }

    private boolean matchRadioAction(ActionEvent e, JRadioButton t) 
    {
	return e.getActionCommand().equals(t.getText());
    }

    private JButton addButton(String name)
    {
	JButton result = new JButton(name);
	result.setHorizontalAlignment(JButton.CENTER);
	result.setActionCommand(result.getText());
	result.addActionListener(this);
	result.setEnabled(false);
	add(result);	
	return result;
    }

    private JRadioButton addRadioButton(String name)
    {
	JRadioButton result = new JRadioButton(name);
	result.setHorizontalAlignment(JRadioButton.CENTER);
	result.setSelected(false);
	result.setEnabled(false);
	result.setActionCommand(result.getText());
	result.addActionListener(this);
	add(result);	
	return result;
    }

    public void actionPerformed(ActionEvent e)
    {
	if(e.getActionCommand().equals(Controls.getResetButton().getText())) {
	    reset();
	}
	else if(e.getActionCommand().equals(eraseAll.getText())) {
	    reset();  // do penup
	    if(sketcher != null)
		sketcher.eraseAll(); // erase all drawn objects
	}
	else if(matchRadioAction(e, lineControl)) {
	    if(sketcher != null)
		sketcher.setLinePenDn(true);
	    penUpControl.setSelected(false);
	    //eraseControl.setSelected(false);
	    lineControl.setSelected(true);
	    circControl.setSelected(false);
	    rectControl.setSelected(false);
	}
	else if(matchRadioAction(e, circControl)) {
	    reset();
	    if(sketcher != null)
		sketcher.setCircPenDn(true);
	    penUpControl.setSelected(false);
	    //eraseControl.setSelected(false);
	    lineControl.setSelected(false);
	    circControl.setSelected(true);
	    rectControl.setSelected(false);
	}
	else if(matchRadioAction(e, rectControl)) {
	    reset();
	    if(sketcher != null)
		sketcher.setRectPenDn(true);
	    penUpControl.setSelected(false);
	    //eraseControl.setSelected(false);
	    lineControl.setSelected(false);
	    circControl.setSelected(false);
	    rectControl.setSelected(true);
	}
	/*
	  else if(matchRadioAction(e, eraseControl)) {
	  sketcher.setErase(true);
	  penUpControl.setSelected(false);
	  eraseControl.setSelected(true);
	  lineControl.setSelected(false);
	  circControl.setSelected(false);
	  rectControl.setSelected(false);
	  }
	*/
	else if(matchRadioAction(e, penUpControl)) {
	    reset();
	}

	updateHighlighting();
    }

    protected void updateHighlighting()
    {
	//penUpControl.setForeground(penUpControl.isSelected() ?
	//Color.black : Color.gray);
	//penDn.setForeground((penUpControl.isSelected() || eraseControl.isSelected()) ?
	penDn.setForeground(penUpControl.isSelected() ?
			    Color.gray : Color.black);
    }

    public void reset()
    {
	Util.dbgPrintln("SketchPanel::reset()");
	System.out.println("SketchPanel::PenUp");
	if(sketcher != null) {
	    //sketcher.setErase(false);
	    sketcher.setCircPenDn(false);
	    sketcher.setRectPenDn(false);
	    sketcher.setLinePenDn(false);
	}
	penUpControl.setSelected(true);
	//eraseControl.setSelected(false);
	rectControl.setSelected(false);
	lineControl.setSelected(false);
	circControl.setSelected(false);	

	updateHighlighting();
    }

    public void enableButtons(boolean v)
    {
	if(v != penUpControl.isEnabled()) {
	    eraseAll.setEnabled(v);
	    penUpControl.setEnabled(v);
	    //eraseControl.setEnabled(v);
	    rectControl.setEnabled(v);
	    lineControl.setEnabled(v);
	    circControl.setEnabled(v);	
	}
    }

    public ArrayList getStates()
    {
	ArrayList result = new ArrayList();
	result.add(new Boolean(eraseAll.isEnabled()));
	result.add(new Boolean(penUpControl.isSelected()));
	result.add(new Boolean(penUpControl.isEnabled()));
	//result.add(new Boolean(eraseControl.isSelected()));
	//result.add(new Boolean(eraseControl.isEnabled()));
	result.add(new Boolean(rectControl.isSelected()));
	result.add(new Boolean(rectControl.isEnabled()));
	result.add(new Boolean(lineControl.isSelected()));
	result.add(new Boolean(lineControl.isEnabled()));
	result.add(new Boolean(circControl.isSelected()));
	result.add(new Boolean(circControl.isEnabled()));
	return result;
    }
}




