/*    
       This software may be used and distributed according to the terms of
       the GNU General Public License (GPL), incorporated herein by
       reference.  Drivers based on this skeleton fall under the GPL and
       must retain the authorship (implicit copyright) notice.
 
       This program is distributed in the hope that it will be useful, but
       WITHOUT ANY WARRANTY; without even the implied warranty of
       MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
       General Public License for more details.
 */
package cricketdaemon.clientlib.parse;

import cricketdaemon.Util;
import cricketdaemon.clientlib.data.*;
import java.io.*;
import java.util.*;

/**
 * Parses subtypes contained within a Packet.
 *
 * @author Allen Miu
 */
public class PacketParseHandlerMap extends ParseHandlerMap
{
    public static PacketParseHandlerMap instance = 
	new PacketParseHandlerMap();

    protected PacketParseHandlerMap()
    {
	super();
	register("array:dist_est", "check_datatype", "parse_adist_est");
	register("array:stat_est", "check_datatype", "parse_astat_est");
	register("device_pos", "check_datatype", "parse_device_pos");
	register("cur_space", "check_datatype", "parse_cur_space");
	register("orient", "check_datatype", "parse_orient");
    }

    public void parse_device_pos(Object dataStructure, String value)
	throws IOException
    {
	StringReader sr = new StringReader(value);
	while(Field.newField(sr)) // new dest_est record
	    Field.parseField(dataStructure,
			     sr, 
			     DevicePosParseHandlerMap.instance);
    }

    public boolean check_datatype(Object o) 
    {
	return o instanceof CricketData;
    }

    public static void parse_adist_est(Object dataStructure, String value)
	throws IOException
    {
	ArrayList beaconsHeard = new ArrayList();
	CricketData cricketData = (CricketData) dataStructure;

	System.err.println("PacketParseHandlerMap::parse_adist_est() do"+
			   " not support data from SpaceModule anymore");
	return;

	/*
	cricketData.setBeaconsHeard(beaconsHeard);

	StringReader sr = new StringReader(value);
	while(Field.newField(sr)) // new dest_est record
	    Field.parseField(beaconsHeard,
			     sr, 
			     DistanceEstimateArrayParseHandlerMap.instance);
	
	// assumes server returns distance estimates in ascending order
	// if not, we can easily add a field (a handler) to specify the 
	// current space object
	cricketData.setCurrentSpaceObject((BeaconRecord) 
					  beaconsHeard.get(0));
	*/
    }

    public void parse_astat_est(Object dataStructure, String value)
	throws IOException
    {
	ArrayList beaconsHeard = new ArrayList();
	CricketData cricketData = (CricketData) dataStructure;

	StringReader sr = new StringReader(value);
	while(Field.newField(sr)) // new dest_est record
	    Field.parseField(beaconsHeard,
			     sr, 
			     DistanceEstimateArrayParseHandlerMap.instance);

	//System.out.println("parsed beacons list "+beaconsHeard.size());
	cricketData.setBeaconsHeard(beaconsHeard);
	cricketData.setBeaconStat();
    }

    public void parse_cur_space(Object dataStructure, String value)
	throws IOException
    {
	ArrayList curspace = new ArrayList(); // for now, this is just a container to parse dist_est
	CricketData cricketData = (CricketData) dataStructure;

	StringReader sr = new StringReader(value);
	while(Field.newField(sr))
	    Field.parseField(curspace,
			     sr, 
			     DistanceEstimateArrayParseHandlerMap.instance);

	cricketData.setCurrentSpaceObject((BeaconRecord) curspace.get(0));
    }

    public void parse_orient(Object dataStructure, String value)
	throws IOException
    {
	CricketData cricketData = (CricketData) dataStructure;
	cricketData.setOrient(Util.parseDouble(value));
    }

}
