/*    
       This software may be used and distributed according to the terms of
       the GNU General Public License (GPL), incorporated herein by
       reference.  Drivers based on this skeleton fall under the GPL and
       must retain the authorship (implicit copyright) notice.
 
       This program is distributed in the hope that it will be useful, but
       WITHOUT ANY WARRANTY; without even the implied warranty of
       MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
       General Public License for more details.
 */
package cricketdaemon.reader.serial;

import cricketdaemon.Util;
import cricketdaemon.module.*;
import java.util.*;

/**
 * Class used to support dynamic class loading for the OldSerialPortReader
 * and SerialPortReader.
 *
 * @author Allen Miu
 */
public abstract class PortReaderCreator
{
    public static Hashtable portReaderFactories = new Hashtable();

    public static final String PATH = "cricketdaemon.reader.";

    // The return type is ModuleThread to prevent the loading
    // of the SerialPortReader class, unless this method is called.
    public static ModuleThread createPortReader(String name,
						String configFile)
    {
	PortReaderFactory p = 
	    (PortReaderFactory) portReaderFactories.get(name);
	if(p == null) {
	    try {
		Class.forName(name);
		p = (PortReaderFactory) portReaderFactories.get(name);
		Util.assert_nonjava(p != null, 
			    "PortReaderCreator::createPortReader()");
	    }
	    catch (ClassNotFoundException e) {
		Util.assert_nonjava(false, "PortReaderCreator::createPortReader() "+
			    name+" class not found");
	    }
	}
	return p.create(configFile);
    }
}
