/*    
       This software may be used and distributed according to the terms of
       the GNU General Public License (GPL), incorporated herein by
       reference.  Drivers based on this skeleton fall under the GPL and
       must retain the authorship (implicit copyright) notice.
 
       This program is distributed in the hope that it will be useful, but
       WITHOUT ANY WARRANTY; without even the implied warranty of
       MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
       General Public License for more details.
 */
package cricketdaemon.server;

import cricketdaemon.*;
import java.net.*;
import java.util.*;
import java.io.*;

/**
 * States of a registered client that wish to obtain updates from the
 * CricketDaemon.
 *
 * @author Allen Miu
 */
class ClientRecord {
    BufferedReader reader = null;

    InetAddress addr;
    int port;
    long expiration;
    Socket s = null;

    int state;

    public final static int CONNECTING  = 0;
    public final static int ESTABLISHED = 1;

    public ClientRecord(Socket sock, long e)
    {
	this(sock.getInetAddress(), sock.getPort(), e);
	s = sock;
    }

    public ClientRecord(InetAddress a, int p, long e)
    {
	addr = a; port = p; expiration = e; state = CONNECTING;
    }
    
    public boolean equals(ClientRecord other)
    {
	return addr.equals(other.addr) && port == other.port;
    }

    static public int indexOf(ClientRecord query, ArrayList a)
    {
	for(int i = 0; i < a.size(); i++) {
	    ClientRecord item = (ClientRecord) a.get(i);
	    if(query.equals(item))
		return i;
	}
	return -1;
    }

    public DatagramPacket createPacket(byte[] payload)
	throws IOException
    {
	Util.assert_nonjava(s == null, "ClientRecord::createPacket()");
	return new DatagramPacket(payload, payload.length, addr, port);
    }

    public Socket getSocket()
    {
	return s;
    }

    public BufferedReader getBufferedReader()
	throws IOException
    {
	if(reader == null)
	    reader = new BufferedReader(new InputStreamReader(getSocket().getInputStream()));
	return reader;
    }

    public String toString()
    {
	return addr.toString() + ":" + port;
    }
}

